@@ -1,8 +1,8 @@
-# @(#) $Id: Atom.pm 1081 2005-11-14 12:59:41Z dom $
+# @(#) $Id: Atom.pm 1102 2005-12-07 14:33:19Z dom $
 
 package Log::Dispatch::Atom;
 
-$VERSION = 0.02;
+$VERSION = 0.03;
 
 use warnings;
 use strict;
@@ -10,10 +10,12 @@ use strict;
 use Carp qw( carp croak );
 use Fcntl qw( :flock );
 use POSIX qw( strftime );
-use Params::Validate;
+use Params::Validate qw( validate SCALAR HASHREF );
+use Sys::Hostname qw( hostname );
 use XML::Atom 0.15;    # We need add_entry(mode=>insert).
 use XML::Atom::Entry;
 use XML::Atom::Feed;
+use XML::Atom::Person;
 
 use base qw( Log::Dispatch::Output );
 
@@ -26,32 +28,51 @@ sub new {
     my %p     = @_;
 
     my $self = bless {}, $class;
+    # Log::Dispatch setup.
     $self->_basic_init( %p );
+    # Our setup.
     $self->_init( %p );
     return $self;
 }
 
 sub _init {
     my $self = shift;
-    my %p = validate( @_, { file => 1, feed_id => 0, feed_title => 0 } );
-    $self->{ file } = $p{ file };
-    $self->{ feed_id } = $p{ feed_id } if $p{ feed_id };
-    $self->{ feed_title } = $p{ feed_title } if $p{feed_title};
+    my %p    = validate(
+        @_,
+        {
+            file        => { type => SCALAR },
+            feed_id     => { type => SCALAR, optional => 1 },
+            feed_title  => { type => SCALAR, optional => 1 },
+            feed_author => { type => HASHREF, optional => 1 },
+        }
+    );
+    $self->{ file }        = $p{ file };
+    $self->{ feed_id }     = $p{ feed_id } if $p{ feed_id };
+    $self->{ feed_title }  = $p{ feed_title } if $p{ feed_title };
+    $self->{ feed_author } = $p{ feed_author } if $p{ feed_author };
     return;
 }
 
-sub _now { strftime "%Y-%m-%dT%H:%M:%SZ", gmtime }
+sub _now_datetime { strftime "%Y-%m-%dT%H:%M:%SZ", gmtime }
+sub _now_date     { strftime "%Y-%m-%d", gmtime }
 
 sub log_message {
     my $self = shift;
-    my %p = @_;
+    my %p    = validate(
+        @_,
+        {
+            id     => { type => SCALAR,  default  => $self->_default_id },
+            author => { type => HASHREF, optional => 1 },
+        }
+    );
     my $fh = eval {
-        my $fh    = $self->_lock_and_open();
-        my $feed  = $self->_get_feed_from_handle( $fh );
+        my $fh   = $self->_lock_and_open();
+        my $feed = $self->_get_feed_from_handle( $fh );
         $self->_new_entry( $feed, \%p );
         $self->_write_feed( $fh, $feed );
         return $fh;
     };
+
     # Take care to avoid clobbering $@.
     my $err = $@;
     eval { $self->_unlock_and_close( $fh ) };
@@ -59,6 +80,15 @@ sub log_message {
     return;
 }
 
+{
+    my $i;
+    sub _default_id {
+        my $self = shift;
+        return sprintf( "tag:%s,%s:%d/%d/%d",
+            hostname(), $self->_now_date(), time(), $$, ++$i );
+    }
+}
+
 sub _get_feed_from_handle {
     my $self = shift;
     my ( $fh ) = @_;
@@ -68,25 +98,47 @@ sub _get_feed_from_handle {
         return XML::Atom::Feed->new( $fh );
     }
     else {
-        # Create a new feed.
-        my $feed = XML::Atom::Feed->new( Version => '1.0' );
-        $feed->id( $self->{ feed_id } ) if $self->{ feed_id };
-        $feed->title( $self->{ feed_title } ) if $self->{ feed_title };
-        return $feed;
+        return $self->_new_feed();
     }
 }
 
+sub _new_feed {
+    my $self = shift;
+    my $feed = XML::Atom::Feed->new( Version => '1.0' );
+    $feed->id( $self->{ feed_id } )       if $self->{ feed_id };
+    $feed->title( $self->{ feed_title } ) if $self->{ feed_title };
+    if ( my $author = $self->{ feed_author } ) {
+        $feed->author( $self->_new_person( $author ) );
+    }
+    return $feed;
+}
+
+sub _new_person {
+    my $self     = shift;
+    my ( $args ) = @_;
+    my $person   = XML::Atom::Person->new( Version => '1.0' );
+    $person->name( $args->{ name } )   if $args->{ name };
+    $person->email( $args->{ email } ) if $args->{ email };
+    $person->uri( $args->{ uri } )     if $args->{ uri };
+    return $person;
+}
+
 sub _new_entry {
     my $self = shift;
     my ( $feed, $args ) = @_;
     my $entry = XML::Atom::Entry->new( Version => '1.0' );
     $entry->title( "$args->{message}" );
     $entry->content( "$args->{message}" );
+    $entry->id( "$args->{id}" );
 
-    my $now = _now();
+    my $now = _now_datetime();
     $feed->updated( $now );
     $entry->updated( $now );
 
+    if ( $args->{ author } ) {
+        $entry->author( $self->_new_person( $args->{ author } ) );
+    }
+
     $feed->add_entry( $entry, { mode => 'insert' } );
     return $entry;
 }
@@ -208,10 +260,26 @@ If not specified, it will be omitted, which is in violation of the
 Atom specification.  For more information, see
 L<http://www.atomenabled.org/developers/syndication/#requiredFeedElements>.
 
-B<XXX> This should probably just use the I<name> parameter.
+B<XXX> This should probably just use the I<name> parameter.  What do you
+think?  Let me know.
+
+=item I<feed_author> [optional]
+
+The author details of a feed.  This is specified as a hash reference,
+which must contain one or more of the three keys I<name>, I<email> and
+I<uri>.
+
+In order to create a valid Atom feed, you must either supply an author
+in every single entry (log message), or ensure that the feed itself has
+an author.  The latter is probably the easier solution, so I recommend
+this parameter be supplied.
 
 =back
 
+B<NB>:  The I<feed_*> parameters will only be used when a new feed is
+being created.  If you are creating a new object for an existing feed,
+they will be ignored.
+
 =item log_message()
 
 Takes a hash of arguments.  Has no return value.  The following
@@ -227,6 +295,23 @@ The actual log message.
 
 The level of the message.  See L<Log::Dispatch> for a full list.
 
+=item I<id> [optional]
+
+Each entry requires an id in order for the feed as a whole to be a valid
+Atom document.  It's used by readers of Atom documents to determine
+whether or not an entry has been seen previously.
+
+If not specified, this will default to an URL comprising the current
+time plus the pid plus the hostname plus a monotonically increasing
+integer.  eg: tag:fred.example.com,2005-12-07:1133946771/20827/2.  This
+should be good enough for a uniqueness test.
+
+=item I<author> [optional]
+
+You can specify author details for an individual entry if desired.  The
+author parameter is expected to be a hash reference, which must contain
+one or more of the keys I<name>, I<email> or I<uri>.
+
 =back
 
 =back
